@@ -38,9 +38,7 @@ class expect(object):
     For more options and a list of the matchers, see the source. :)
     """
     
-    # TODO: Consider to add with statement support
-    # with expect.raises(KeyError, 'fnord'):
-    #     something_that_raises()
+    # REFACT: consider moving private methods down so reading the source allows immediate access to the matchers
     
     def __init__(self, expected, should_raise=True, message=None):
         """Initialize a useable assertion object that you can chain off of.
@@ -60,6 +58,8 @@ class expect(object):
         self._expected_assertion_result = True
         self._selected_matcher = None
         self._selected_matcher_name = None
+        
+        self._enable_nicer_backtraces_for_new_double_underscore_matcher_alternatives()
     
     @classmethod
     def with_message(cls, message, expected, should_raise=True):
@@ -87,106 +87,6 @@ class expect(object):
         """
         return cls(expected, should_raise=False, message=message)
     
-    ## Internals ########################################################################################
-    
-    def _is_negative(self):
-        return self._expected_assertion_result is False
-    
-    def __getattribute__(self, name):
-        """Allows you to chain any python identifier to this object and keep 
-        chaining for as long as you want as syntactic sugar.
-        
-        Only the last identifier that is then actually called has to be an actual 
-        matcher.
-        
-        Including 'not' in any of the identifiers used in chaining will invert the 
-        whole expectation.
-        """
-        
-        # Allow normal access to everything that is not a matcher. 
-        # (Every public method is treated as a matcher)
-        # REFACT: I would prefer it if not every access to private methods would go through this method
-        # However as I do not want to return instance methods and other existing methods directly, there
-        # seems to be no way to do this.
-        if name.startswith('_'):
-            return object.__getattribute__(self, name)
-        
-        # If you include not somewhere in the called attributes name, 
-        # switch the expectation to negative.
-        # REFACT: consider to support calling negated matchers directly with something like: expect(3).not_equals(2)
-        if name.startswith('not_') or '_not_' in name or name.endswith('_not'):
-            self._expected_assertion_result = False
-        
-        # If a matcher is availeable switch to it to allow calling it
-        self._selected_matcher_name = name
-        if hasattr(self.__class__, name):
-            self._selected_matcher = object.__getattribute__(self, name)
-        else:
-            self._selected_matcher = None
-        
-        # Allow arbitrary chaining
-        return self
-    
-    def __call__(self, *args, **kwargs):
-        """Called whenever you actualy invoke a matcher. 
-        
-        Provides a good error message if you mistype the matcher name.
-        
-        Supports custom messages with the message keyword argument"""
-        # Integration with py.test - this hides the __call__ method from the generated traceback
-        __tracebackhide__ = True
-        
-        if self._selected_matcher is None:
-            # REFACT: consider raising NotImplementedError
-            raise AssertionError("Tried to call non existing matcher '{}' (Patches welcome!)".format(self._selected_matcher_name))
-        
-        # Make the stacktrace easier to read by tricking python to shorten the stack trace to this method.
-        # Hides the actual matcher and all the methods it calls to assert stuff.
-        try:
-            self._selected_matcher(*args, **kwargs)
-        except AssertionError as assertion:
-            if self._should_raise:
-                # hide internal expect() methods from backtrace
-                raise assertion
-            
-            # Support returning_expect
-            return (False, str(assertion))
-        
-        return (True, "")
-    
-    # REFACT: would be nice if the name of this method makes it clear how the 
-    # message should be worded to give a good error message
-    def _message(self, message_format, *message_positionals, **message_keywords):
-        expected = self._expected
-        optional_negation = ' not ' if self._expected_assertion_result is False else ' '
-        message = message_format.format(*message_positionals, **message_keywords)
-        assertion_message = "Expect {expected!r}{optional_negation}{message}".format(
-            expected=expected,
-            optional_negation=optional_negation,
-            message=message,
-        )
-        
-        if self._custom_message is not None:
-            return self._custom_message.format(**locals())
-        
-        return assertion_message
-    
-    def _assert(self, assertion, message_format, *message_positionals, **message_keywords):
-        assert assertion is self._expected_assertion_result, \
-            self._message(message_format, *message_positionals, **message_keywords)
-    
-    def _assert_if_positive(self, assertion, message_format, *message_positionals, **message_keywords):
-        if self._expected_assertion_result is False:
-            return
-        
-        self._assert(assertion, message_format, *message_positionals, **message_keywords)
-    
-    def _assert_if_negative(self, assertion, message_format, *message_positionals, **message_keywords):
-        if self._expected_assertion_result is True:
-            return
-        
-        self._assert(assertion, message_format, *message_positionals, **message_keywords)
-    
     ## Matchers #########################################################################################
     
     # On naming matchers: Their name should be clear and fit in with the naming scheme of the existing 
@@ -196,7 +96,7 @@ class expect(object):
     
     # On debugging matchers: Some pyton debuggers will hide all the internals of the expect method
     # To match py.tests behaviour. Read up on hidden frames and how to unhide them in your python debugger
-    # `hf_unhide` is ofthen the keyword here.
+    # `hf_unhide` is often the keyword here.
     
     def is_true(self):
         self._assert(self._expected is True, "to be True")
@@ -209,15 +109,17 @@ class expect(object):
     false = is_false
     
     def is_equal(self, something):
-        self._assert(something == self._expected, "to be equal to {}", something)
+        self._assert(something == self._expected, "to be equal to {!r}", something)
     
     __eq__ = equals = equal = to_equal = is_equal
     
-    def __ne__(self, something):
+    def is_different(self, something):
         self.not_ == something
     
+    __ne__ = is_different
+    
     def to_be(self, something):
-        self._assert(something is self._expected, "to be {}", something)
+        self._assert(something is self._expected, "to be {!r}", something)
     
     is_ = be = be_same = is_same = be_identical = is_identical = to_be
     
@@ -231,10 +133,11 @@ class expect(object):
     
     falsy = falsish = falseish = is_falseish
     
-    def does_include(self, something):
-        self._assert(something in self._expected, "to include {}", something)
+    def does_include(self, *needles):
+        for needle in needles:
+            self._assert(needle in self._expected, "to include {!r}", needle)
     
-    contain = contains = include = includes = does_include
+    contain = contains = to_include = include = includes = does_include
     
     def is_included_in(self, sequence_or_atom, *additional_atoms):
         sequence = sequence_or_atom
@@ -242,7 +145,7 @@ class expect(object):
             sequence = [sequence_or_atom]
             sequence.extend(additional_atoms)
         
-        self._assert(self._expected in sequence, "is included in {}", sequence)
+        self._assert(self._expected in sequence, "is included in {!r}", sequence)
     
     in_ = included_in = is_included_in
     
@@ -258,17 +161,20 @@ class expect(object):
         expected_items = [(key, self._expected.get(key)) for key in actual_keys]
         # superset = set(self._expected.iteritems())
         # REFACT: subset.issubset(superset)
-        self._assert(expected_items == actual_items, 'to contain dict {}', a_subdict)
+        self._assert(expected_items == actual_items, 'to contain dict {!r}', a_subdict)
     
     includes_dict = contains_dict = sub_dict = subdict = have_subdict = have_sub_dict = has_subdict = has_sub_dict
     
     def to_match(self, regex):
         assert isinstance(self._expected, basestring), self._message("to be a string")
         
-        self._assert(re.search(regex, self._expected) is not None, "to be matched by regex r'{}'", regex)
+        self._assert(re.search(regex, self._expected) is not None, "to be matched by regex r{!r}", regex)
     
     match = matches = is_matching = to_match
     
+    # TODO: consider with statement support to allow code like this
+    # with expect.raises(AssertionError):
+    #   something.that_might_raise()
     def to_raise(self, exception_class=Exception, message_regex=None):
         """Be carefull with negative raise assertions as they swallow all exceptions that you 
         don't specify. If you say `expect(raiser).not_.to_raise(FooException, 'fnord')` this 
@@ -304,7 +210,7 @@ class expect(object):
     def is_instance(self, a_class):
         self._assert(isinstance(self._expected, a_class), "to be instance of '{}'", a_class.__name__)
     
-    instanceof = instance_of = isinstance = is_instance
+    instanceof = is_instance_of = instance_of = isinstance = is_instance
     
     def has_length(self, a_length):
         self._assert(len(self._expected) == a_length, "to have length {}", a_length)
@@ -312,35 +218,181 @@ class expect(object):
     len = length = count = has_count = has_length
     
     def is_greater(self, smaller):
-        self._assert(self._expected > smaller, "to be greater then {}", smaller)
+        self._assert(self._expected > smaller, "to be greater then {!r}", smaller)
     
     __gt__ = bigger = bigger_then = larger = larger_then = is_greater_then = is_greater
     
     def is_greater_or_equal(self, smaller_or_equal):
-        self._assert(self._expected >= smaller_or_equal, "to be greater or equal then {}", smaller_or_equal)
+        self._assert(self._expected >= smaller_or_equal, "to be greater or equal then {!r}", smaller_or_equal)
     
     __ge__ = greater_or_equal_then = is_greater_or_equal_then = greater_or_equal = is_greater_or_equal
     
     def is_less_then(self, greater):
-        self._assert(self._expected < greater, "to be less then {}", greater)
+        self._assert(self._expected < greater, "to be less then {!r}", greater)
     
     __lt__ = smaller = smaller_then = lesser = lesser_then = less_then = smaller_then = is_smaller_then = is_less_then
     
     def is_less_or_equal(self, greater_or_equal):
-        self._assert(self._expected <= greater_or_equal, "to be less or equal then {}", greater_or_equal)
+        self._assert(self._expected <= greater_or_equal, "to be less or equal then {!r}", greater_or_equal)
     
     __le__ = smaller_or_equal = smaller_or_equal_then = is_smaller_or_equal = is_smaller_or_equal_then = is_less_or_equal_then = is_less_or_equal
     
-    # REFACT: consider adding is_within_exclusive_range
+    # TODO: consider adding is_within_exclusive_range
+    # TODO: consider supporting slice syntax as alias. expect(3)[2:4] doesn't look natural though
     def is_within_range(self, lower, higher):
-        self._assert(lower <= self._expected <= higher, "to be between {} and {}", lower, higher)
+        self._assert(lower <= self._expected <= higher, "to be between {!r} and {!r}", lower, higher)
     
     is_between = is_within = is_within_range
     
     def is_close(self, actual, delta):
-        self._assert((actual - delta) <= self._expected <= (actual + delta), "to be close to {} with max delta {}", actual, delta)
+        self._assert((actual - delta) <= self._expected <= (actual + delta), "to be close to {!r} with max delta {!r}", actual, delta)
     
     almost_equal = is_almost_equal = close_to = is_close_to = is_close
+    
+    ## Internals ########################################################################################
+    
+    def __getattribute__(self, name):
+        """Allows you to chain any python identifier to this object and keep 
+        chaining for as long as you want as syntactic sugar.
+        
+        Only the last identifier that is then actually called has to be an actual 
+        matcher.
+        
+        Including 'not' in any of the identifiers used in chaining will invert the 
+        whole expectation. This includes calling the negated form of a matcher by 
+        prefixing it with not_ like 'not_to_be' to get the inverted form of 'to_be'.
+        """
+        # Allow normal access to everything that is not a matcher. 
+        # (Every public method is treated as a matcher)
+        # REFACT: I would prefer it if not every access to private methods would go through this method
+        # However as I do not want to return instance methods and other existing methods directly, there
+        # seems to be no way to do this.
+        if name.startswith('_'):
+            return object.__getattribute__(self, name)
+        
+        # If you include not somewhere in the called attributes name, 
+        # switch the expectation to negative.
+        if name.startswith('not_') or '_not_' in name or name.endswith('_not'):
+            self._expected_assertion_result = False
+        
+        matcher = self._matcher_with_name(name)
+        self._prepare_matcher_for_calling(name, matcher)
+        
+        # Allow arbitrary chaining
+        return self
+    
+    def _matcher_with_name(self, name):
+        # REFACT: consider to change matcher lookup to allow some more pre- and suffixes
+        # could be be_ to_ and some more to dry up the list of alternative names that have 
+        # to be written out for each matcher.
+        if name.startswith('not_'):
+            name = name[4:]
+        
+        # hasattr on __class__ to prevent recursion from __getattribute__
+        if not hasattr(self.__class__, name):
+            return None
+        
+        return object.__getattribute__(self, name)
+        
+    def _prepare_matcher_for_calling(self, name, matcher):
+        self._selected_matcher_name = name
+        self._selected_matcher = matcher
+    
+    def __call__(self, *args, **kwargs):
+        """Called whenever you actualy invoke a matcher. 
+        
+        Provides a good error message if you mistype the matcher name.
+        
+        Supports custom messages with the message keyword argument"""
+        __tracebackhide__ = True  # Hide from py.test tracebacks
+        
+        if self._selected_matcher is None:
+            # REFACT: consider raising NotImplementedError
+            raise AssertionError("Tried to call non existing matcher '{}' (Patches welcome!)".format(self._selected_matcher_name))
+        
+        # Make the stacktrace easier to read by tricking python to shorten the stack trace to this method.
+        # Hides the actual matcher and all the methods it calls to assert stuff.
+        try:
+            self._selected_matcher(*args, **kwargs)
+        except AssertionError as assertion:
+            if self._should_raise:
+                # hide internal expect() methods from backtrace
+                raise assertion
+            
+            # Support returning_expect
+            return (False, str(assertion))
+        
+        return (True, "")
+    
+    def _assert(self, assertion, message_format, *message_positionals, **message_keywords):
+        assert assertion is self._expected_assertion_result, \
+            self._message(message_format, *message_positionals, **message_keywords)
+    
+    def _assert_if_positive(self, assertion, message_format, *message_positionals, **message_keywords):
+        if self._is_negative():
+            return
+        
+        self._assert(assertion, message_format, *message_positionals, **message_keywords)
+    
+    def _assert_if_negative(self, assertion, message_format, *message_positionals, **message_keywords):
+        if not self._is_negative():
+            return
+        
+        self._assert(assertion, message_format, *message_positionals, **message_keywords)
+    
+    # REFACT: would be nice if the name of this method makes it clear how the 
+    # message should be worded to give a good error message
+    def _message(self, message_format, *message_positionals, **message_keywords):
+        expected = self._expected
+        optional_negation = ' not ' if self._is_negative() else ' '
+        message = message_format.format(*message_positionals, **message_keywords)
+        assertion_message = "Expect {expected!r}{optional_negation}{message}".format(
+            expected=expected,
+            optional_negation=optional_negation,
+            message=message,
+        )
+        
+        if self._custom_message is not None:
+            return self._custom_message.format(**locals())
+        
+        return assertion_message
+    
+    def _is_negative(self):
+        return self._expected_assertion_result is False
+    
+    @classmethod
+    def _enable_nicer_backtraces_for_new_double_underscore_matcher_alternatives(cls):
+        """Works around the problem that special methods (as in '__something__') are not resolved
+        using __getattribute__() and thus do not provide the nice tracebacks that they public matchers provide.
+        
+        This method works on the class dict and is repeated after each instantiation
+        to support adding or overwriting existing matchers at any time while not repeatedly doing the same.
+        """
+        # TODO: try if turning the special methods into descriptors allows to trigger
+        # self.__getattribute__() without increasing the stack trace length on failures
+        def wrap(special_method_name, public_name, matcher):
+            def wrapper(self, *args, **kwargs):
+                __tracebackhide__ = True  # Hide from py.test tracebacks
+                # Sadly this increases the traceback lenght by one entry :/
+                self.__getattribute__(public_name)(*args, **kwargs)
+            setattr(cls, special_method_name, wrapper)
+        
+        special_names = filter(lambda name: name.startswith('__') and name.endswith('__'), dir(cls))
+        public_matchers_by_matcher = dict()
+        for name in dir(cls):
+            # Assumes that all public methods are matchers
+            if not name.startswith('_'):
+                matcher = object.__getattribute__(cls, name)
+                public_matchers_by_matcher[matcher] = name
+        
+        for special_name in special_names:
+            matcher = object.__getattribute__(cls, special_name)
+            if matcher in public_matchers_by_matcher.keys():
+                public_name = public_matchers_by_matcher[matcher]
+                wrap(special_name, public_name, matcher)
+    
+
+## Unit Tests ###########################################################################################
 
 from unittest import TestCase, main
 class ExpectTest(TestCase):
@@ -380,13 +432,40 @@ class ExpectTest(TestCase):
         expect(lambda: expect.with_message('fnord', True).to.be(False)) \
             .to_raise(AssertionError, r"^fnord$")
     
-    def test_should_allow_to_formulate_abstract_expectations(self):
+    def test_should_allow_to_formulate_non_raising_expectations(self):
         # Idea: have a good api to check expectations without raising
         expect(expect(False, should_raise=False).to_be(False)).equals((True, ""))
         expect(expect(False, should_raise=False).not_to.be(True)).equals((True, ""))
         expect(expect.returning(False).to_be(False)).equals((True, ""))
         expect(expect.returning(False).to_be(True)).to_equal((False, "Expect False to be True"))
     
+    def test_should_hide_double_underscore_alternative_names_from_tracebacks(self):
+        assertion = None
+        try:
+            expect(3) != 3
+        except AssertionError as a:
+            assertion = a
+        
+        expect(assertion) != None
+        import sys, traceback
+        traceback = '\n'.join(traceback.format_tb(sys.exc_info()[2]))
+        expect(traceback).not_to.contain('_assert')
+        # Would like this one too, but is only hidden in py.test
+        # expect(traceback).not_to.contain('self(')
+    
+    def test_should_allow_not_variety_of_every_matcher_directly(self):
+        expect(3).to_be(3)
+        expect(3).not_to_be(2)
+        expect(lambda: expect(3).not_to_be(3)) \
+            .to_raise(AssertionError, r"^Expect 3 not to be 3$")
+        
+        expect(lambda: None).not_to_raise(AssertionError)
+        raising = lambda: expect(lambda: 1 / 0).not_to_raise(ZeroDivisionError)
+        expect(raising).to_raise(AssertionError, "integer division or modulo by zero")
+    
+    def _test_should_give_good_error_message_when_missing_argument_to_expect(self):
+        pass
+    
     # Matchers
     
     def test_is_trueish(self):
@@ -484,8 +563,10 @@ class ExpectTest(TestCase):
         expect([1,2,3,4]).include(3)
         expect([23,42]).not_to.contain(7)
         expect(dict(foo='bar')).includes('foo')
+        expect([1,2,3,4]).includes(2,3)
         
         expect(lambda: expect([1,2]).to.contain(3)).to_raise(AssertionError, r"Expect \[1, 2] to include 3")
+        expect(lambda: expect([1,2]).to_include(2,3)).to_raise(AssertionError, r"Expect \[1, 2] to include 3")
     
     def test_has_subdict(self):
         expect(dict()).to_have.subdict()
